@@ -5,7 +5,7 @@ fn main() -> anyhow::Result<()> {
     let pylon = pylon_cxx::Pylon::new();
 
     // Create an instant camera object with the camera device found first.
-    let camera = pylon_cxx::TlFactory::instance(&pylon).create_first_device()?;
+    let mut camera = pylon_cxx::TlFactory::instance(&pylon).create_first_device()?;
 
     // Print the model name of the camera.
     println!("Using device {:?}", camera.device_info().model_name()?);
