@@ -1,3 +1,7 @@
+#if defined(FEATURE_STREAM)
+#include <unistd.h>
+#include <fcntl.h>
+#endif
 #include <memory>
 #include "pylon/PylonIncludes.h"
 #include "pylon-cxx-rs.h"
@@ -139,6 +143,12 @@ namespace Pylon
         return camera->IsGrabbing();
     }
 
+    #if defined(FEATURE_STREAM)
+    int instant_camera_wait_object_fd(const std::unique_ptr<CInstantCamera> &camera) {
+      return camera->GetGrabResultWaitObject().GetFd();
+    }
+    #endif
+
     bool instant_camera_retrieve_result(const std::unique_ptr<CInstantCamera> &camera, uint32_t timeout, std::unique_ptr<CGrabResultPtr> &result, TimeoutHandling timeout_handling)
     {
         ETimeoutHandling eth;
